@@ -1,4 +1,7 @@
 
+#   不要在此文件使用任何缩进格式化功能！
+#   不要在此文件使用任何缩进格式化功能！
+#   不要在此文件使用任何缩进格式化功能！
 
 model_header = """
 from pydantic import BaseModel
@@ -37,14 +40,15 @@ class {{ actionsName }}(Actions):
 
     def connection(self, data={}):
         #   write your code
-        {% if connetionKeys %}{% for key in connetionKeys %}
+        {% if connectionKeys %}{% for key in connectionKeys %}
         {{ key }} = data.get("{{ key }}"){% endfor %}{% else %}
         ...    
         {% endif %}
     
     def run(self, params={}):
         #   write your code
-        #   可调用self._popEmpty()去除载荷中的空参数
+        #   可调用 popEmpty() 去除载荷中的空参数
+        #   使用 setLocalCache() 设置本地缓存
         {% if inputKeys %}{% for key in inputKeys %}
         {{ key }} = params.get("{{ key }}"){% endfor %}{% else %}
         ...
@@ -72,14 +76,16 @@ class {{ triggersName }}(Triggers):
 
     def connection(self, data={}):
         #   write your code
-        {% if connetionKeys %}{% for key in connetionKeys %}
+        {% if connectionKeys %}{% for key in connectionKeys %}
         {{ key }} = data.get("{{ key }}"){% endfor %}{% else %}
         ...    
         {% endif %}
 
     def run(self, params={}):
         #   write your code
-        #   可调用self._popEmpty()去除载荷中的空参数
+        #   可调用 popEmpty() 去除载荷中的空参数
+        #   使用 self.setCache() 设置在线缓存
+        #   使用 setLocalCache() 设置本地缓存
         #   返回必须使用 self.send({})
         {% if inputKeys %}{% for key in inputKeys %}
         {{ key }} = params.get("{{ key }}"){% endfor %}{% else %}
@@ -109,15 +115,17 @@ class {{ alarm_receiversName }}(AlarmReceivers):
 
     def connection(self, data={}):
         #   write your code
-        {% if connetionKeys %}{% for key in connetionKeys %}
+        {% if connectionKeys %}{% for key in connectionKeys %}
         {{ key }} = data.get("{{ key }}"){% endfor %}{% else %}
         ...    
         {% endif %}
 
     def run(self, params={}):
         #   write your code
-        #   可调用self._popEmpty()去除载荷中的空参数
+        #   可调用 popEmpty() 去除载荷中的空参数
         #   返回必须使用 self.send({})
+        #   使用 self.setCache() 设置在线缓存
+        #   使用 setLocalCache() 设置本地缓存
         {% if inputKeys %}{% for key in inputKeys %}
         {{ key }} = params.get("{{ key }}"){% endfor %}{% else %}
         ...
@@ -147,14 +155,16 @@ class {{ indicator_receiversName }}(IndicatorReceivers):
 
     def connection(self, data={}):
         #   write your code
-        {% if connetionKeys %}{% for key in connetionKeys %}
+        {% if connectionKeys %}{% for key in connectionKeys %}
         {{ key }} = data.get("{{ key }}"){% endfor %}{% else %}
         ...    
         {% endif %}
 
     def run(self, params={}):
         #   write your code
-        #   可调用self._popEmpty()去除载荷中的空参数
+        #   可调用 popEmpty() 去除载荷中的空参数
+        #   使用 self.setCache() 设置在线缓存
+        #   使用 setLocalCache() 设置本地缓存
         #   返回必须使用 self.send({})
         {% if inputKeys %}{% for key in inputKeys %}
         {{ key }} = params.get("{{ key }}"){% endfor %}{% else %}
@@ -164,56 +174,302 @@ class {{ indicator_receiversName }}(IndicatorReceivers):
 """
 
 
+action_rest_template = """
+from SDK.subassembly import Actions
+from SDK.base import *
+
+import requests
+
+from .models import {{ connModel }}, {{ inputModel }},{{ outputModel }}
+
+import requests
+import base64
+import tempfile
+
+class {{ actionsName }}(Actions):
+
+    def __init__(self):
+        #   初始化
+        super().__init__()
+        self.name = "{{ name }}"
+        self.inputModel = {{ inputModel }}
+        self.outputModel = {{ outputModel }}
+        self.connModel = {{ connModel }}
+
+    def connection(self, data={}):
+        #   write your code
+        {% for key in connection_url_params_keys %}
+        self.{{ key[0] }} = data.get("{{ key[0] }}"){% endfor %}
+        {% for key in connection_headers_keys %}
+        self.{{ key[0] }} = data.get("{{ key[0] }}"){% endfor %}
+        {% for key in connection_params_keys %}
+        self.{{ key[0] }} = data.get("{{ key[0] }}"){% endfor %}
+        {% for key in connection_body_keys %}
+        self.{{ key[0] }} = data.get("{{ key[0] }}"){% endfor %}
+        {% for key in connection_files_keys %}
+        self.{{ key[0] }} = data.get("{{ key[0] }}"){% endfor %}
+        {% if connection_ssl_verify %}
+        self.{{ connection_ssl_verify[0] }} = data.get("{{ connection_ssl_verify[0] }}"){% endif %}
+        ...
+
+    
+    def run(self, params={}):
+        #   write your code
+        #   可调用 popEmpty() 去除载荷中的空参数
+        #   使用 setLocalCache() 设置本地缓存
+        {% for key in run_url_params_keys %}
+        self.{{ key[0] }} = params.get("{{ key[0] }}"){% endfor %}
+        {% for key in run_headers_keys %}
+        self.{{ key[0] }} = params.get("{{ key[0] }}"){% endfor %}
+        {% for key in run_params_keys %}
+        self.{{ key[0] }} = params.get("{{ key[0] }}"){% endfor %}
+        {% for key in run_body_keys %}
+        self.{{ key[0] }} = params.get("{{ key[0] }}"){% endfor %}
+        {% for key in run_files_keys %}
+        self.{{ key[0] }} = params.get("{{ key[0] }}"){% endfor %}
+        {% if run_ssl_verify %}
+        self.{{ run_ssl_verify[0] }} = data.get("{{ run_ssl_verify[0] }}"){% endif %}
+
+        self.url = f"{{ url }}"
+
+        _params = {}
+        {% if run_params_keys %}{% for key in run_params_keys %}
+        if self.{{ key[0] }}:
+            _params["{{ key[1] }}"] = self.{{ key[0] }}
+            {% endfor %}{% endif %}  
+        {% if connection_params_keys %}{% for key in connection_params_keys %}
+        if self.{{ key[0] }}:
+            _params["{{ key[1] }}"] = self.{{ key[0] }}
+        {% endfor %}{% endif %}
+
+        headers = {}
+        {% if run_headers_keys %}{% for key in run_headers_keys %}
+        if self.{{ key[0] }}:
+            headers["{{ key[1] }}"] = self.{{ key[0] }}
+        {% endfor %}{% endif %}
+        {% if connection_headers_keys %}{% for key in connection_headers_keys %}
+        if self.{{ key[0] }}:
+            headers["{{ key[1] }}"] = self.{{ key[0] }}
+        {% endfor %}{% endif %}
+
+        {% if body_type == None %}
+        payload = {}
+        {% endif %}
+
+        {% if body_type == "JSON" %}
+        payload = {}
+        {% if run_body_keys %}{% for key in run_body_keys %}
+        if self.{{ key[0] }}:
+            payload["{{ key[1] }}"] = self.{{ key[0] }}
+        {% endfor %}{% endif %}
+        {% if connection_body_keys %}{% for key in connection_body_keys %}
+        if self.{{ key[0] }}:
+            payload["{{ key[1] }}"] = self.{{ key[0] }}
+        {% endfor %}{% endif %}
+        payload = json.dumps(payload)
+        {% endif %}
+
+        {% if body_type == "x-www-form-urlencoded" %}
+        payload = ""
+        {% if run_body_keys %}{% for key in run_body_keys %}
+        if self.{{ key[0] }}:
+            payload += f"{{ key[1] }}={self.{{ key[0] }}}&"
+        {% endfor %}{% endif %}
+        {% if connection_body_keys %}{% for key in connection_body_keys %}
+        if self.{{ key[0] }}:
+            payload += f"{{ key[1] }}={self.{{ key[0] }}}&"
+        {% endfor %}{% endif %}
+        if len(payload) > 1:
+            payload = payload[:-1]
+        {% endif %}
+
+        {% if body_type == "form-data" %}
+        payload = {}
+        {% if run_body_keys %}{% for key in run_body_keys %}
+        if self.{{ key[0] }}:
+            payload["{{ key[1] }}"] = self.{{ key[0] }}
+        {% endfor %}{% endif %}
+        {% if connection_body_keys %}{% for key in connection_body_keys %}
+        if self.{{ key[0] }}:
+            payload["{{ key[1] }}"] = self.{{ key[0] }}
+        {% endfor %}{% endif %}
+        {% endif %}
+
+        files = []
+        {% if run_files_keys %}{% for key in run_files_keys %}
+        if self.{{ key[0] }}:
+            file_bytes = base64.b64decode(self.{{ key[0] }}.get("content",""))
+            {{ key[0] }}_tempfile = tempfile.TemporaryFile()
+            {{ key[0] }}_tempfile.write(file_bytes)
+            {{ key[0] }}_tempfile.seek(0)
+            if file_bytes:
+                files.append(("{{ key[1] }}",(self.{{ key[0] }}.get("filename",""),{{ key[0] }}_tempfile)))
+        {% endfor %}{% endif %}
+        {% if connection_files_keys %}{% for key in connection_files_keys %}
+        if self.{{ key[0] }}:
+            file_bytes = base64.b64decode(self.{{ key[0] }}.get("content",""))
+            {{ key[0] }}_tempfile = tempfile.TemporaryFile()
+            {{ key[0] }}_tempfile.write(file_bytes)
+            {{ key[0] }}_tempfile.seek(0)
+            if file_bytes:
+                files.append(("{{ key[1] }}",(self.{{ key[0] }}.get("filename",""),{{ key[0] }}_tempfile)))
+        {% endfor %}{% endif %}
+
+        try: 
+            response = requests.request("{{ method }}", self.url, headers=headers, data=payload, {% if run_ssl_verify %}verify=self.{{ run_ssl_verify[0] }}, {% endif %}{% if connection_ssl_verify %}verify=self.{{ connection_ssl_verify[0] }}, {% endif %}params=_params, files=files)
+        except Exception as error:
+            raise Exception(f"请求失败 - {error}")
+
+        {% if run_files_keys %}{% for key in run_files_keys %}{{ key[0] }}_tempfile.close()
+        {% endfor %}{% endif %}
+        {% if connection_files_keys %}{% for key in run_files_keys %}{{ key[0] }}_tempfile.close()
+        {% endfor %}{% endif %}
+
+        try:
+            #   尝试获取返回数据
+            response_data = response.json()
+        except:
+            response_data = {}
+
+        return {
+            "code" : response.status_code,
+            "response_data" : response_data,
+            "response_text" : response.text
+        }
+"""
+
+
 actions_test_template = """
 {
-	"version": "v1",
-	"type": "action_start",
-	"body": {
-		"action": "{{ title }}",
-		"meta": {},
-		"connection": {},
-		"dispatcher": null,
-		"input": {}
-	}
+  "version": "v3",
+  "type": "action_start",
+  "body": {
+    "action": "{{ name }}",
+    "meta": {},
+    "connection": {
+    },
+    "dispatcher": null,
+    "input": {
+    },
+    "config": {
+      "log": {
+        "max_log_size": 2,
+        "max_log_size_unit": "MB",
+        "log_clear_size": 0.5,
+        "debug": false
+      },
+      "thread": {
+        "threads_limit": 5
+      }
+    }
+  }
 }
-
 """
 
 triggers_test_template = """
 {
-	"version": "v1",
-	"type": "trigger_start",
-	"body": {
-		"trigger": "{{ title }}",
-		"meta": {},
-		"connection": {},
-		"dispatcher": {
-			"url": "http://127.0.0.1:8001/send",
-			"webhook_url": ""
-		},
-		"input": {},
-      "enable_web": false
-	}
+  "version": "v3",
+  "type": "trigger_start",
+  "body": {
+    "trigger": "{{ name }}",
+    "meta": {},
+    "connection": {
+    },
+    "dispatcher": {
+      "url": "http://127.0.0.1:10001/transpond",
+      "cache_url": ""
+    },
+    "input": {
+    },
+    "enable_web": false,
+    "config": {
+      "log": {
+        "max_log_size": 2,
+        "max_log_size_unit": "MB",
+        "log_clear_size": 0.5,
+        "debug": false
+      },
+      "thread": {
+        "threads_limit": 5
+      }
+    }
+  }
 }
+"""
 
+alarm_receivers_test_template = """
+{
+  "version": "v3",
+  "type": "alarm_receiver_start",
+  "body": {
+    "alarm": "{{ name }}",
+    "meta": {},
+    "connection": {
+    },
+    "dispatcher": {
+      "url": "http://127.0.0.1:10001/transpond",
+      "cache_url": ""
+    },
+    "input": {
+    },
+    "enable_web": false,
+    "config": {
+      "log": {
+        "max_log_size": 2,
+        "max_log_size_unit": "MB",
+        "log_clear_size": 0.5,
+        "debug": false
+      },
+      "thread": {
+        "threads_limit": 5
+      }
+    }
+  }
+}
+"""
+
+indicator_receivers_test_template = """
+{
+  "version": "v3",
+  "type": "indicator_receiver_start",
+  "body": {
+    "receiver": "{{ name }}",
+    "meta": {},
+    "connection": {
+    },
+    "dispatcher": {
+      "url": "http://127.0.0.1:10001/transpond",
+      "cache_url": ""
+    },
+    "input": {
+    },
+    "enable_web": false,
+    "config": {
+      "log": {
+        "max_log_size": 2,
+        "max_log_size_unit": "MB",
+        "log_clear_size": 0.5,
+        "debug": false
+      },
+      "thread": {
+        "threads_limit": 5
+      }
+    }
+  }
+}
 """
 
 main_template = """#!/usr/bin/env python
 
 from SDK.cli import client
-
 {% if actionClassees %}
-import actions
-{% endif %}
+import actions{% endif %}
 {% if triggerClassees %}
-import triggers
-{% endif %}
+import triggers{% endif %}
 {% if indicatorReceiverClassees %}
-import indicator_receivers
-{% endif %}
+import indicator_receivers{% endif %}
 {% if alarmReceiverClassees %}
-import alarm_receivers
-{% endif %}
+import alarm_receivers{% endif %}
 
 # 整个程序入口
 
@@ -270,8 +526,7 @@ if __name__ == '__main__':
     
 """
 
-init_template = \
-"""{% for name, className in init_list %}
+init_template = """{% for name, className in init_list %}
 from .{{ name }} import {{ className }}
 {% endfor %}"""
 
@@ -493,26 +748,6 @@ None
 ## 参考引用
 """
 
-
-indicator_receivers_test_template = """
-{
-	"version": "v1",
-	"type": "indicator_receiver_start",
-	"body": {
-		"receiver": "{{ title }}",
-		"meta": {},
-		"connection": {},
-		"dispatcher": {
-			"url": "http://127.0.0.1:8001/send",
-			"webhook_url": ""
-		},
-		"input": {},
-      "enable_web": false
-	}
-}
-
-"""
-
 indicator_receivers_model_types = """
 class Indicator(BaseModel):
     uid: str
@@ -616,26 +851,6 @@ class {{ className }}(BaseModel):
 
 """
 
-
-alarm_receivers_test_template = """
-{
-	"version": "v1",
-	"type": "alarm_receiver_start",
-	"body": {
-		"alarm": "{{ title }}",
-		"meta": {},
-		"connection": {},
-		"dispatcher": {
-			"url": "http://127.0.0.1:8001/send",
-			"webhook_url": ""
-		},
-		"input": {},
-      "enable_web": false
-	}
-}
-
-"""
-
 alarm_receivers_model_types = """
 class Alarm(BaseModel):
     uid: str
@@ -670,14 +885,14 @@ from SDK.base import *
 
 description = \
 '''
-  欢迎使用 1.2.7 版本SDK提供的全新测试系统。\n
+  欢迎使用 1.2.8 版本SDK提供的全新测试系统。\n
   相较于之前的版本的 REST 测试接口，这个版本提供了更加详细的功能细分以及测试数据输入指引。\n
   现在再也不会几个接口测试一大堆功能，一大堆参数还不知道怎么填了。\n
   
   Enjoy it!  -- Matthews_K
 
 '''
-test_server = FastAPI(title="Chariot-Plugin Test Server", version="1.2.7", description=description)
+test_server = FastAPI(title="Chariot-Plugin Test Server", version="1.2.8", description=description)
 
 """
 
@@ -694,10 +909,9 @@ if __name__ == '__main__':
     runserver()
 """
 
-testserver_actions_template="""
+testserver_actions_template = """
 @test_server.post("/actions/{{ name }}",response_model={{ class_name }}().outputModel,tags=["动作"])
-def action_{{ name }}(action_name:str="{{ name }}",
-                      connection_data:{{ class_name }}().connModel=None,
+def action_{{ name }}(connection_data:{{ class_name }}().connModel=None,
                       input_data:{{ class_name }}().inputModel=None):
     
     clearLog()
@@ -717,10 +931,9 @@ def action_{{ name }}(action_name:str="{{ name }}",
 
 """
 
-testserver_triggers_template="""
+testserver_triggers_template = """
 @test_server.post("/triggers/{{ name }}",response_model={{ class_name }}().outputModel,tags=["触发器"])
-def trigger_{{ name }}(trigger_name:str="{{ name }}",
-                       dispatcher_url:str="http://127.0.0.1:8000/send",
+def trigger_{{ name }}(dispatcher_url:str="http://127.0.0.1:8000/send",
                        connection_data:{{ class_name }}().connModel=None,
                        input_data:{{ class_name }}().inputModel=None):
     
@@ -741,10 +954,9 @@ def trigger_{{ name }}(trigger_name:str="{{ name }}",
 
 """
 
-testserver_alarm_receivers_template="""
+testserver_alarm_receivers_template = """
 @test_server.post("/alarm_receivers/{{ name }}",response_model={{ class_name }}().outputModel,tags=["告警接收器"])
-def alarm_receiver_{{ name }}(alarm_receiver_name:str="{{ name }}",
-                              dispatcher_url:str="http://127.0.0.1:8000/send",
+def alarm_receiver_{{ name }}(dispatcher_url:str="http://127.0.0.1:8000/send",
                               connection_data:{{ class_name }}().connModel=None,
                               input_data:{{ class_name }}().inputModel=None):
     
@@ -765,10 +977,9 @@ def alarm_receiver_{{ name }}(alarm_receiver_name:str="{{ name }}",
 
 """
 
-testserver_indicator_receivers_template="""
+testserver_indicator_receivers_template = """
 @test_server.post("/indicator_receivers/{{ name }}",response_model={{ class_name }}().outputModel,tags=["情报接收器"])
-def indicator_receiver_{{ name }}(indicator_receiver_name:str="{{ name }}",
-                              dispatcher_url:str="http://127.0.0.1:8000/send",
+def indicator_receiver_{{ name }}(dispatcher_url:str="http://127.0.0.1:8000/send",
                               connection_data:{{ class_name }}().connModel=None,
                               input_data:{{ class_name }}().inputModel=None):
     
@@ -787,4 +998,6 @@ def indicator_receiver_{{ name }}(indicator_receiver_name:str="{{ name }}",
 
     return output_data
 
-"""
\ No newline at end of file
+"""
+
+
